;Data locking section here...
LockInit:
	ld (hl),0
	ret

LockToggle:
	ld a,(hl)
	xor 1					;
	ld (hl),a
	ret

LockResolve:
	ld a,(hl)				;check, locked?
	cp 0
	jp z,LockResolveUL
	ld hl,0
	ret
LockResolveUL:
	ld (hl),1				;lock it
	inc hl
	ret						;and return the location

LockState:
	ld a,(hl)
	ret

LockWait:
	ld a,(hl)				;check, locked?
	cp 1
	jp z,LockWait			;loop until it's unlocked
	di
	ld a,(hl)				;incase an interupt happened during this, which locked it before we got to it*grumble grumble*
	cp 1
	jp z,LockWaitResume
	call LockResolve		;Resolve it, reinable interupts, and return
	ei
	ret
LockWaitResume:
	ei						;enable interupts so we arn't stuck here forever...
	jp LockWait				;and resume the waiting!
	
;multi Threading section here...
ThreadDefault:		.dw 0
ThreadFirst:		.dw 0
ThreadLast:			.dw 0
ThreadCurrent:		.dw 0
ThreadAddNextTemp:	.dw 0
Thread_Temp:		.dw 0
Thread_Temp2:		.dw 0
UserCounter:		.dw 0

ThreadInit:
	pop bc
	ld hl,0
	add hl,sp
	ld (ThreadDefault),hl
	push bc

	ld hl, ThreadFlip			;Load the Thread system...
	ld de, $8f8f
	ld bc, Thread_end-Thread_Start
	ldir

	ld hl, $8e00			;Load up the vector table....
	ld de, $8e01
	ld (hl), $8f
	ld bc,257
	ldir

	ld a,$8E
	ld i,a
	im 2

#ifndef THREAD_NO_GREY
	exx
	ld b,$3c
	ld c,0
	ld d,3
	ld e,%110110
	ld hl,UserCounter
	exx
#endif
	ret


ThreadQuit:
	di
	ld hl,(ThreadDefault)	;restores the default stack
	ld sp,hl

#ifndef THREAD_NO_GREY
	ld a,$3C				;make sure video mem is active
	out (0),a

	call ClearScreen

#endif
	im 1
	ei
	ret


ThreadStart:
	ld hl,(ThreadFirst)
	jp ThreadGetNext


ClearScreen:				;clear video mem and graph mem
	call _clrLCD
#ifndef THREAD_NO_GREY
clrGrf:						;clear graph mem
	ld hl,_plotSScreen+6
	ld d,h
	ld e,l
	inc de
	ld (hl), 0
	ld bc,1024
	ldir
#endif
	ret


ThreadAdd:
	push hl
	push de
	ld hl,(ThreadCurrent)
	ld a,h
	cp 0
	jp nz,ThreadAddNext

	ld (ThreadCurrent),de
	ld (ThreadFirst),de
	ld (ThreadLast),de

ThreadAddNext:
	pop ix
	pop hl

	ld (ix-1),h				;save the location...
	ld (ix-2),l

	ld hl,-ThreadMinStack+1	;save the start of the stack....
	add hl,de
	ld (ix-13),h
	ld (ix-14),l

	ld hl,(ThreadLast)		;save the last table value
	ld (ix-17),h
	ld (ix-18),l

	ld hl,(ThreadFirst)		;save the next table value
	ld (ix-15),h
	ld (ix-16),l

	push hl					;pull up the first table, and set it's last table value to the added table
	pop ix

	ld (ix-17),d
	ld (ix-18),e

	ld hl,(ThreadLast)		;pull up the 'last table' and set it's next table value to the added table
	push hl
	pop ix

	ld (ix-15),d
	ld (ix-16),e

	ld hl,0
	add hl,sp
	ld (ThreadAddNextTemp),hl

	ld hl,-ThreadTableSize+1;load the threads stack
	add hl,de
	ld sp,hl

	push de					;push the table location
	ld hl,ThreadDeleteThis
	push hl					;push the entrance point...

	ld hl,(ThreadAddNextTemp)
	ld sp,hl				;restore stack...

	ld (ThreadLast),de

	ret						;leave


ThreadDelete:
	push hl
	jp ThreadDeleteThis


ThreadDeleteThis:
	di
	pop ix
	ld h,(ix-15)				;hl has the next table location
	ld l,(ix-16)
	ld d,(ix-17)				;de has the last table location
	ld e,(ix-18)
	push ix
	pop bc
	ld a,b						;is the next table the same table as this one?
	cp h
	jp nz,ThreadDeleteNoQuit
	ld a,c
	cp l
	jp z,ThreadQuit				;quit
ThreadDeleteNoQuit:
	push hl						;pull up the 'next table'
	pop ix
	ld (ix-17), d				;save the 'last table' item in the new table, as the previous table
	ld (ix-18), e
	push de
	pop ix
	ld (ix-15), h				;save the 'next table' item in the table pointed to by the 'last table' of the current table as the next table
	ld (ix-16), l

	ld hl,(ThreadCurrent)		;is the current table this table?
	ld a,b
	cp h
	jp nz,ThreadDeleteEnd
	ld a,c
	cp l
	jp nz,ThreadDeleteEnd

	jp ThreadGetNext			;execute the 'get next' table...
ThreadDeleteEnd:
	ei							;resume interupts
	ret							;bye bye


ThreadFlip:
.org $8f8f

Thread_Start:
	nop
	ld (Thread_Temp),ix			;save ix...
	ld ix,(ThreadCurrent)

	ld (ix-5),b					;save bc to the thread table
	ld (ix-6),c

	ld (ix-7),d					;save de to the thread table
	ld (ix-8),e

	ld (ix-9),h					;save hl to the thread table
	ld (ix-10),l

	ld hl,(Thread_Temp)			;save ix to the thread table
	ld (ix-11),h
	ld (ix-12),l

	pop bc						;save where we were
	ld (ix-1),b
	ld (ix-2),c

	push af						;save af to the thread table
	pop bc
	ld (ix-3),b
	ld (ix-4),c

	ld hl,0						;save the stack to the table
	add hl,sp
	ld (ix-13),h
	ld (ix-14),l

	ld h,(ix-15)				;set hl to the next table
	ld l,(ix-16)

	ld (ThreadCurrent),hl
ThreadGetNext:
	push hl
	pop ix

	ld h,(ix-3)					;restore af
	ld l,(ix-4)
	push hl
	pop af

	ld b,(ix-5)					;restore bc
	ld c,(ix-6)
	ld d,(ix-7)					;restore de
	ld e,(ix-8)
	ld h,(ix-9)					;restore hl
	ld l,(ix-10)

	ld (Thread_Temp),hl			;save hl

	ld h,(ix-13)				;restore the stack...
	ld l,(ix-14)
	ld sp,hl

	ld h,(ix-1)					;restore where we were
	ld l,(ix-2)
	push hl

	ld h,(ix-11)
	ld l,(ix-12)
	ld (Thread_Temp2),hl
	ld ix,(Thread_Temp2)

	ld hl,(Thread_Temp)			;finally restore the stack
#ifndef THREAD_NO_GREY
	exx						;We exchange the variables, and let the default interupt run, before 'resuming'
	ex af,af'

	inc (hl)			;inc user counter
	out (c),b
	dec d			
	call z,reset_int_counter
	ld a,d
	cp 1
	call z,change_pages	;flip pages every once and a while
leave_int:
	in a,(3)			;this stuff must be done or calc crashes
	rra				;mysterious stuff from the ROM
	ld a,c
	adc a,9
	out (3),a
	ld a,$0B
	out (3),a
	ex af,af'
	exx
	ei
	reti
reset_int_counter:	;reset int counter d when it is 0
	ld d,3
change_pages:
	ld a,e
	xor b			;toggle memories
	ld b,a
	ret
#else
	ei
	reti
#endif
Thread_end:
