#include "ti86asm.inc"
#include "Thread.inc"

.org _asm_exec_ram

	nop
	jp ProgStart
	.dw 0
	.dw ShellTitle
ShellTitle:
	.db "Simple Threading",0

ProgStart:
	call ThreadInit	;Initalize the threads..

	ld hl,main
	ld de,End_of_Main_Stack
	call ThreadAdd

	ld hl,Thread2
	ld de,End_of_Thread2_Stack
	call ThreadAdd		;add the 2 threads used in the program...
	di

	ld hl,SomeVar2		;initalize the locking system for this variable
	call LockInit

	call ClearScreen

	call ThreadStart

main:				;loop loop loop loop
	call RenderSprite
	ld hl,SomeVar2
	call LockWait

	ld a,(hl)

	cp $ff
	jp z,Leave			;this check, checks to see if it's time to leave or not...

	ld hl,SomeVar2
	call LockToggle		;reinable the memory once you've used it for whatever you need it for.

	ld ix,Main_Stack	;you can now modify ix inside of a function + some amount to be a 
	call FakeLocalVar
	jp main
Main_Stack:	.pad ($+ ThreadMinStack + 100)		;have 100 avalible bytes for the stack.
End_of_Main_Stack:

FakeLocalVar:
	ld a,(ix+0)			; ix+0 is a variable located in memory, that will be unique to each Thread, if you load ix to the start of the threads stack.
	inc a
	ld (ix+0),a
	ret

RenderSprite:
	ld hl, $FC80		; HL points to display
	ld de, sprite		; DE points to start of sprite
	ld b, 8

Loop:
	ld a, (de)          ; Get byte from sprite
	ld (hl), a          ; Display byte on screen

	inc de              ; Next byte of sprite

	ld a, l
	add a, 16           ; Add 16 to L (goes to next row)
	ld l, a
        
	djnz Loop

	ret
Leave:
	call ThreadQuit		;Shutdown Threads..
	ret					;Leave the program

Thread2:
	ld hl,SomeVar2		;Simple incrementation of a locked variable

	call LockWait		;wait for it to be free...

	ld (Thread2_Temp),hl
	ld b,(hl)			;load the value to de
	inc b				;increment it
	ld (hl),b			;store it

	ld hl,SomeVar2
	call LockToggle		;unlock the variable

	ld ix,Thread2_Stack	;you can now modify ix inside of a function + some amount to be a unique local variable
	call FakeLocalVar
	jp Thread2
Thread2_Temp:	.dw 0
Thread2_Stack:	.pad ($+ ThreadMinStack + 100)		;have 100 avalible bytes for the stack.
End_of_Thread2_Stack:


SomeVar2: .pad ($+ LockedMinSize + 2)				;we need 2 bytes for this variable...

sprite:
	.db %01100000
	.db %01110000
	.db %00100100
	.db %01111011
	.db %10110100
	.db %01110000
	.db %00101000
	.db %01101100
#include "Thread.asm"

.end